import { HostedMode } from "./HostedMode";
import { WidgetPermission } from "../permission/permission";
import { LanguageCode, LanguageTextMap } from "../lang/LanguageCode";
type WidgetOptions = {
    name: string;
    title: {
        [key: string]: string;
    };
    description: {
        [key: string]: string;
    };
    keywords: WidgetKeyword[];
    lang: string;
    width: number;
    height: number;
    maxWidth?: number;
    maxHeight?: number;
    minWidth?: number;
    minHeight?: number;
    packageName?: string;
    previewImage?: string;
    /**
     * 悬浮窗模式 是否可移动
     */
    movable?: boolean;
    supportHostedMode?: HostedMode;
    backgroundThrottling?: boolean;
    shortcut?: string;
    /**
     * 组件拉伸后，是否强制刷新组件
     */
    refreshWhenResided?: boolean;
    hideWhenBlur?: boolean;
    routes: WidgetRoute[];
    permissions?: WidgetPermission[];
    broadcastChannels?: string[];
};
export declare class Widget {
    readonly name: string;
    /**
     * 组件标题，显示在界面上的，
     * https://zh.m.wikipedia.org/zh-hans/ISO_639-1
     */
    readonly title: LanguageTextMap;
    /**
     * 组件介绍
     */
    readonly description: LanguageTextMap;
    readonly keywords: WidgetKeyword[];
    /**
     * 组件默认语言
     */
    readonly lang: string;
    readonly width: number;
    packageName?: string | null;
    readonly height: number;
    readonly maxWidth: number;
    readonly maxHeight: number;
    readonly minWidth: number;
    readonly minHeight: number;
    readonly movable: boolean;
    readonly backgroundThrottling: boolean;
    /**
     * 预览图片，可以是GIF
     */
    readonly previewImage?: string;
    readonly supportHostedMode: number;
    readonly hideWhenBlur: boolean;
    readonly shortcut?: string;
    /**
     * 组件拉伸后，是否强制刷新组件
     */
    readonly refreshWhenResided: boolean;
    /**
     * 组件其他页面的url在这注册
     */
    readonly routes: WidgetRoute[];
    readonly permissions: WidgetPermission[];
    constructor(options: WidgetOptions);
    getIndexRoute(): WidgetRoute;
    getConfigRoute(): WidgetRoute | undefined;
    isConfigurable(): boolean;
    /**
     * 获取组件标题
     * @param lang 语言环境，不传则获取默认语言
     */
    getTitle(lang?: LanguageCode): string | undefined;
    /**
     * 获取组件描述
     * @param lang 语言环境，不传则获取默认标题
     */
    getDescription(lang?: LanguageCode): string | undefined;
    static parseJSON(json: string): Widget;
    static parseObject(obj: any): Widget;
    /**
     * 是否支持悬浮窗
     */
    isSupportOverlap(): boolean;
    isResizable(): boolean;
    isSupportBackground(): boolean;
    /**
     * 是否支持普通模式
     */
    isSupportNormal(): boolean;
    isSupportWallpaper(): boolean;
    isSupportScreen(): boolean;
}
export interface WidgetRoute {
    name: string;
    url: string;
    meta?: {
        [key: string]: string;
    };
    /**
     * 使用自定义悬浮窗样式
     */
    customOverlapView?: boolean;
    windowOptions?: WindowOption;
}
export interface WindowOption {
    width?: number;
    height?: number;
    maxHeight?: number;
    maxWidth?: number;
    minWidth?: number;
    minHeight?: number;
}
export declare enum WidgetKeyword {
    RECOMMEND = "recommend",
    TOOLS = "tools",
    EFFICIENCY = "efficiency",
    PICTURE = "picture",
    LIFE = "life",
    SHORTCUT = "shortcut",
    COUNTDOWN = "countdown",
    TIMER = "timer",
    INFO = "info",
    DASHBOARD = "dashboard"
}
export {};
